unit MMTimer;

interface

uses
 MMSystem,
  SysUtils,
  Classes;

type

 EMMTimerError = class (Exception);
 TMMEventMode = (mmeOneShort, mmePeriodic);

 TMMTimer = class (TComponent)
  private
   FMinPeriod, FMaxPeriod: LongWord;
   FDelay, FResolution: LongWord;
   FTimerID: LongWord;
   FEventMode: TMMEventMode;
   FonTimer: TNotifyEvent;
   FEnabled: boolean;
   procedure SetDelay(Delay: LongWord);
   procedure SetResolution(Resolution: LongWord);
   procedure SetMode(NewMode: TMMEventMode);
   procedure SetEnabled(NewState: boolean);
   procedure SetOnTimer(NEvent: TNotifyEvent);
   procedure UpdateTimer;
  public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   property MinPeriod: LongWord read FMinPeriod;
   property MaxPeriod: LongWord read FMaxPeriod;
  published
   property Enabled: boolean read FEnabled write SetEnabled default True;
   property Delay: LongWord read FDelay write SetDelay default 1000;
   property Resolution: LongWord read FResolution write SetResolution default 100;
   property Mode: TMMEventMode read FEventMode write SetMode default mmePeriodic;
   property OnTimer: TNotifyEvent read FonTimer write SetOnTimer;
 end;

 procedure Register;

implementation

resourcestring
 SMMTError1 = 'Can''t get DevCaps for MMTimer';
 SMMTError2 = 'Can''t start multimedia timer';

procedure Register;
begin
  RegisterComponents('Samples', [TMMTimer]);
end;

procedure MMTimerProc
 (ID, ReservedMsg, UserData, Reserved1, Reserwed2: LongWord); stdcall;
begin
 if (TObject(UserData) is TMMTimer) and
    Assigned(TMMTimer(UserData).onTimer)
  then TMMTimer(UserData).onTimer(TObject(UserData));
end;

constructor TMMTimer.Create(AOwner: TComponent);
var
 TimeCaps: TTimeCaps;
begin
 inherited Create(AOwner);

 if timeGetDevCaps(@TimeCaps, sizeof(TTimeCaps))<>TIMERR_NOERROR
  then raise EMMTimerError.Create(SMMTError1);

 with TimeCaps do
 begin
  FMinPeriod:=wPeriodMin; FMaxPeriod:=wPeriodMax;
 end;

 FDelay:=1000;
 FResolution:=100;

 FEventMode:=mmePeriodic;
 FEnabled:=True;
end;

destructor TMMTimer.Destroy;
begin
 FEnabled:=False;
 UpdateTimer;
 inherited Destroy;
end;

procedure TMMTimer.SetDelay(Delay: LongWord);
begin
 if FDelay <> Delay
 then begin
  FDelay:=Delay;
  UpdateTimer;
 end;
end;

procedure TMMTimer.SetResolution(Resolution: LongWord);
begin
 if FResolution <> Resolution
 then begin
  FResolution:=Resolution;
  UpdateTimer;
 end;
end;

procedure TMMTimer.SetMode(NewMode: TMMEventMode);
begin
 if FEventMode <> NewMode
 then begin
  FEventMode:=NewMode;
  UpdateTimer;
 end;
end;

procedure TMMTimer.SetEnabled(NewState: boolean);
begin
 if FEnabled <> NewState
 then begin
  FEnabled:=NewState;
  UpdateTimer;
 end;
end;

procedure TMMTimer.SetOnTimer(NEvent: TNotifyEvent);
begin
 FonTimer:=NEvent;
 UpdateTimer;
end;

procedure TMMTimer.UpdateTimer;
begin
 if FTimerID <> 0
 then begin                 // Stop
  timeKillEvent(FTimerID); FTimerID:=0;
 end;

 if (FDelay <> 0) and
     FEnabled and
     Assigned(FOnTimer)
 then begin                 // Start
  FTimerID:=TimeSetEvent(
    FDelay,
    FResolution,
    @MMTimerProc,
    LongWord(Self),
    LongWord(FEventMode)
   );
  if FTimerID=0
   then raise EMMTimerError.Create(SMMTError2);
 end;
end;

end.

